#ifndef lint
static const char rcsid[] =
    "@(#) $Id: fit.c 42 2010-04-17 18:00:29Z leres $ (LBL)";
#endif

/*
 * fit - fit data to a straight line
 *
 * The algorithms come from Numerical Recipes (1986)
 * by Press, Flannery, Teukolsky, and Vetterling.
 *
 */

#include <math.h>

#include "fit.h"

/*
 * Given a set of ndata points x[i], y[i] with standard deviations
 * sig[i], fit them to a straight line y = a * x + b by minimizing x^2.
 * Returned are A, b and their respective probable uncertainties siga
 * and sigb, the chi-square chi2, and the goodness-of-fit probability q
 * (that the fit would have x^2 this large or larger). if mwt == 0 on
 * input, then the standard deviations are assumed to be unavailable: q
 * is returned as 1.0 and the normalization of chi2 is to unit standard
 * deviation on all points.
 */
/*
 * Compute y = a + x * b (and the correlation coefficient r).
 */
void
fit(double *x, double *y, int ndata, double *a, double *b, double *r)
{
	register int i;
	register double sx, sy, sxx, syy, sxy;
	double ss, sdev;

	/* Accumulate sums */
	sx = sxx = sy = syy = sxy = 0.0;
	for (i = ndata; --i >= 0; ) {
		sx += *x;
		sxx += *x * *x;
		sy += *y;
		syy += *y * *y;
		sxy += *x++ * *y++;
	}

	/* Calculate standard deviation */
	ss = ndata;
	sdev = ss * sxx - sx * sx;

	/* Solve for a, b, and r */
	*a = (sxx * sy - sx * sxy) / sdev;
	*b = (ss * sxy - sx * sy) / sdev;
	*r = (ss * sxy - (sy * sx)) / sqrt(sdev * (ss * syy - sy * sy));
}
