#ifndef lint
static const char rcsid[] =
    "@(#) $Id: rdtst.c 54 2010-05-12 00:47:45Z leres $ (LBL)";
#endif
/*
 * readtest file tsize size - do "tsize" kbytes reads of size "size" bytes
 * from "file".
 * report the average time and individual times.
 */
#include <sys/time.h>
#include <sys/file.h>
#include <sys/fcntl.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "version.h"

/* Globals */
const char *prog;

/* Forwards */
int main(int, char **);
void usage(void) __attribute__((noreturn));

int
main(int argc, char **argv)
{
	size_t bufsize;
	int fd;
	int op;
	int ntimes;
	// struct timeval *optime;
	struct timeval start, stop, delta;
	char *buf;
	double delt;
	int verbose = 0;
	int secs = 60;
	const char *file;
	char *cp;

	if ((cp = strrchr(argv[0], '/')) != NULL)
		prog = cp + 1;
	else
		prog = argv[0];

	while ((op = getopt(argc, argv, "vs:")) != EOF)
		switch (op) {

		case 'v':
			++verbose;
			break;

		case 's':
			secs = atoi(optarg);
			if (secs < 1)
				usage();
			break;

		default:
			usage();
		}

	if (argc != optind + 2)
		usage();

	file = argv[optind];

	if ((fd = open(file, O_RDONLY, 0666)) < 0) {
		perror(file);
		exit(1);
	}
	if ((bufsize = atoi(argv[optind + 1])) <= 0) {
		usage();
		exit(4);
	}
	// ntimes = tsize * (1 << 10) / bufsize;
	if (! (buf = malloc(bufsize))) {
		perror ("malloc:");
		exit(5);
	}
#ifdef notdef
	if (! (optime = (struct timeval *)calloc(ntimes + 1, sizeof(struct timeval)))) {
		perror("calloc:");
		exit(5);
	}
#endif
	gettimeofday(&start, NULL);
	ntimes = 0;
	do {
		++ntimes;
		if (read(fd, buf, bufsize) != bufsize) {
			perror("read");
			exit(6);
		}
		gettimeofday(&stop, NULL);
		timersub(&stop, &start, &delta);
	} while (delta.tv_sec < secs);

	delt = (double)delta.tv_sec + 1.0e-6 * (double)delta.tv_usec;
	printf("%d reads\t%d bytes\t%.06f sec\t%.1f ms-avg\t%.0f KB/s\n",
		 ntimes,
		 (u_int)bufsize,
		 delt,
		 delt / (double)ntimes * 1000.,
		 (double)(ntimes * bufsize) / delt / 1024.0);

#ifdef notdef
	if (verbose)
		for (i = 0; ++i <= ntimes; ) {
			delt = (double)(optime[i].tv_sec -
				optime[i - 1].tv_sec) +
				1.0e-6 *
				(double)(optime[i].tv_usec -
				optime[i - 1].tv_usec);

				printf("%d\t%g\n", i - 1, delt);
		}
#endif
	fflush(stdout);
	exit(0);
}

void
usage(void)
{

	fprintf(stderr, "%s version %s\n", prog, version);
	fprintf(stderr, "usage: %s [-d] [-s secs] file blocksize\n",
	    prog);
	exit(1);
}
